/*
 * Licensed Materials - Property of HCL
 * UrbanCode Deploy
 * (c) Copyright HCL Technologies Ltd. 2018. All Rights Reserved.
 */
package com.urbancode.air.plugin.ispw.events

import com.google.gson.JsonObject
import groovy.json.JsonSlurper
import groovy.util.logging.Log

/*
 * Callback event for successful/unsuccessful ISPW operations.
 */
class Event {
    /* Fields that will be serialized to JSON */
    String name // The name of this event (Completed, Failed, Terminated, or Deleted).
    String url  // The URL that will receive the HTTP callback request.
    String method   // The HTTP method to use for the callback request.
    Map<String, Object> body    // The JSON payload to send with the callback request.

    public Event(String name, String url, String method, String payload) {
        this.name = name
        this.method = method
        encodeUrl(url)
        parsePayload(payload)
    }

    /* Encode the callback url query parameters */
    String encodeUrl(String url) {
        /* Split off the query from the rest of the URL */
        url.tokenize("?").with { parts ->
            if (parts[1]) {
                String query = "?"
                List<String> queryParams = parts[1].tokenize("&")
                queryParams.each { queryParam ->
                    if (queryParam.contains("=")) {
                        queryParam.tokenize("=").with { keyVal ->
                            query += keyVal[0] + "=" + URLEncoder.encode(keyVal[1], "UTF-8")
                            if (queryParam != queryParams.last()) {
                                query += "&"
                            }
                        }
                    }
                    else {
                        query += queryParam[0]
                    }
                }

                this.url = parts[0] + query
            }
        }
    }

    /* Parse a JSON string into a map to be used as the 'body' field of the event callback */
    void parsePayload(String payload) {
        if (payload) {
            JsonSlurper slurper = new JsonSlurper()
            body = slurper.parseText(payload)
        }
    }
}
