/*
 * Licensed Materials - Property of HCL
 * UrbanCode Deploy
 * (c) Copyright HCL Technologies Ltd. 2018. All Rights Reserved.
 */
package com.urbancode.air.plugin.ispw.models

import com.google.gson.Gson
import com.google.gson.GsonBuilder
import com.google.gson.JsonElement
import com.google.gson.JsonObject
import org.apache.log4j.Logger

/*
 * Base model for all ISPW REST API informational models (SetInfo, TaskInfo, ReleaseInfo)
 */
class InfoModel {
    /* Fields that will not be serialized to JSON. */
    protected transient Map<String, String> additionalProperties
    protected transient Logger logger

    /* Return the Json representation of this class using GSON. */
    public String getJson() {
        logger = Logger.getLogger(getClass())
        Gson builder = new GsonBuilder().disableHtmlEscaping().create()

        /* Serialize this class instance to JSON */
        JsonElement jsonTree = builder.toJsonTree(this)
        JsonObject jsonObject = jsonTree.getAsJsonObject()

        for (Map.Entry<String, String> entry : additionalProperties) {
            jsonObject.addProperty(entry.getKey(), entry.getValue())
        }

        return jsonObject.toString()
    }

    /* Obtain newline delimited list of key=value pairs. */
    public void parseAdditionalProps(String unparsedProps) {
        additionalProperties = new HashMap<String, String>()

        for (String line : unparsedProps.tokenize("\n|,")) {
            line.tokenize('=').with {
                if (it.size() > 1) {
                    additionalProperties.put(it[0], it[1])
                }
                else {
                    logger.warn("Property entry '${line}' is not in 'key=value' format and will be ignored.")
                }
            }
        }
    }

}
