/*
 * Licensed Materials - Property of HCL
 * UrbanCode Deploy
 * (c) Copyright HCL Technologies Ltd. 2018. All Rights Reserved.
 */
package com.urbancode.air.plugin.ispw.models

import java.util.Map

import com.urbancode.air.plugin.ispw.events.Event
import com.urbancode.air.plugin.ispw.events.EventStatuses
import com.urbancode.air.plugin.ispw.http.HttpHeader

/*
 * This class holds fields to be serialized in a JSON body to ISPW Set related REST endpoints.
 * Any properties left as null will not be serialized to JSON.
 */
class SetInfo extends InfoModel {
    String changeType
    String executionStatus
    String autoDeploy
    String runtimeConfiguration
    Map<String, String> credentials
    List<Event> events
    List<HttpHeader> httpHeaders

    public void setChangeType(String changeType) {
        this.changeType = changeType
    }

    public void setExecutionStatus(String executionStatus) {
        this.executionStatus = executionStatus
    }

    public void setAutoDeploy(String autoDeploy) {
        this.autoDeploy = autoDeploy
    }

    public void setRuntimeConfiguration(String runtimeConfiguration) {
        this.runtimeConfiguration = runtimeConfiguration
    }

    /* Create an event for each given event status */
    public void generateEvents(
        List<String> statuses,
        String eventUrl,
        String eventMethod,
        String eventPayload)
    {
        if (events == null) {
            events = new ArrayList<Event>()
        }

        for (String status in statuses) {
            Event event = new Event(status, eventUrl, eventMethod, eventPayload)
            events.add(event)
        }

    }

    void setCredentials(String username, String password) {
        if (username && password) {
            credentials = new HashMap<String, String>()
            credentials.put("username", username)
            credentials.put("password", password)
        }
    }

    void parseHttpHeaders(String unparsedHeaders) {
        httpHeaders = new ArrayList<HttpHeader>()
        for (String line : unparsedHeaders.tokenize("\n|,")) {
            line.tokenize('=').with {
                httpHeaders.add(new HttpHeader(it[0].trim(), it[1].trim()))
            }
        }
    }
}
