/*
 * Licensed Materials - Property of HCL
 * UrbanCode Deploy
 * (c) Copyright HCL Technologies Ltd. 2018. All Rights Reserved.
 */
package com.urbancode.air.plugin.ispw.versions

import com.urbancode.air.plugin.ispw.models.TaskInfo
import org.apache.http.HttpResponse

/*
 * Client to make requests against all ISPW assignment endpoints (promote, deploy, regress, generate, add tasks).
 */
class AssignmentClient extends VersionClient {
    private final String PROMOTE_ASSIGN_URI = "/ispw/%s/assignments/%s/tasks/promote?level="
    private final String DEPLOY_ASSIGN_URI = "/ispw/%s/assignments/%s/tasks/deploy?level="
    private final String REGRESS_ASSIGN_URI = "/ispw/%s/assignments/%s/tasks/regress?level="
    private final String GENERATE_TASKS_URI = "/ispw/%s/assignments/%s/tasks/generate?level="
    private final String ADD_TASK_URI = "/ispw/%s/assignments/%s/task/add?checkout="

    public AssignmentClient(Properties props) {
        super(props)
    }

    /* Add a task with an optional checkout to an assignment. */
    public Map<String, String> addTask(String id, String checkout) {
        TaskInfo taskInfo = getTaskInfo()
        String uri = String.format(ADD_TASK_URI, serverId, id) + URLEncoder.encode(checkout, "UTF-8")
        HttpResponse response = restHelper.doPostRequest(uri, taskInfo.getJson())
        Map<String, String> responseMap = restHelper.parseResponse(response)
        return responseMap
    }

    /* Acquire a TaskInfo object used for adding assignment tasks. */
    private TaskInfo getTaskInfo() {
        /* Required TaskInfo properties used for adding a task. */
        String application = props['application']?.trim()
        String checkoutFromLevel = props['checkoutFromLevel']?.trim()
        String path = props['path']?.trim()
        String stream = props['stream']?.trim()
        String taskName = props['taskName']?.trim()
        String type = props['type']?.trim()

        /* Optional properties */
        String action = props['action']?.trim()
        String owner = props['owner']?.trim()
        String releaseId = props['releaseId']?.trim()
        String runtimeConfig = props['runtimeConfig']?.trim()

        TaskInfo taskInfo = new TaskInfo(application, checkoutFromLevel, path, stream, taskName, type)

        /* Only configure optional fields if they exist. */
        if (action) {
            taskInfo.setAction(action)
        }
        if (owner) {
            taskInfo.setOwner(owner)
        }
        if (releaseId) {
            taskInfo.setReleaseId(releaseId)
        }
        if (runtimeConfig) {
            taskInfo.setRuntimeConfiguration(runtimeConfig)
        }

        return taskInfo
    }

    @Override
    protected String getPromoteUri() {
        return PROMOTE_ASSIGN_URI
    }

    @Override
    protected String getDeployUri() {
        return DEPLOY_ASSIGN_URI
    }

    @Override
    protected String getRegressUri() {
        return REGRESS_ASSIGN_URI
    }

    @Override
    protected String getGenerateUri() {
        return GENERATE_TASKS_URI
    }

}
